#include "wallpaperrotator2.h"

WallpaperRotator2::WallpaperRotator2(QWidget *parent)
    : QWidget(parent)
{

	ui.setupUi(this);
	emplacements = new ListeEmplacements();
	int i = 0;
	while (true) {
	    QString path = Parametres::getEmplacementPath(i);
	    if (path == "") {
	        break;
	    }
	    emplacements->ajouterEmplacement(path, Parametres::getEmplacementFiltre(i));
	    emplacements->setSelection(i, Parametres::isEmplacementSelected(i));
	    i++;
	}
	ui.tableView->setModel(emplacements);

	QIcon icon(":/WallpaperRotator2.png");
	setWindowIcon(icon);
    setWindowFlags(Qt::Dialog);

	trayIcon = new QSystemTrayIcon(this);
	trayIcon->setIcon(icon);
	connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(on_trayIconAction(QSystemTrayIcon::ActivationReason)));
    trayIcon->show();

    rotator = new Rotator();
    rotator->listerFichiers(emplacements);

    trayMenu = NULL;
    creerTrayMenu();

    connect(ui.boutonAjouter, SIGNAL(clicked()), this, SLOT(on_ajouterEmplacement()));
    connect(ui.boutonSupprimer, SIGNAL(clicked()), this, SLOT(on_supprimerEmplacement()));
}

WallpaperRotator2::~WallpaperRotator2()
{
    delete emplacements;
    delete rotator;
    delete trayIcon;
    delete trayMenu;
}

void WallpaperRotator2::closeEvent(QCloseEvent *event) {
    hide();
    event->ignore();
}

void WallpaperRotator2::creerTrayMenu() {
    // TODO : optimiser pour ne pas tout recrer  chaque fois
    delete trayMenu;
    trayMenu = new QMenu();

    // Menu racine
    QAction* trayMenuChanger = new QAction("Changer", trayMenu);
    trayMenu->addAction(trayMenuChanger);
    connect(trayMenuChanger, SIGNAL(triggered()), rotator, SLOT(changerWallpaper()));

    QAction* trayMenuActiver = new QAction("Activer", trayMenu);
    trayMenuActiver->setCheckable(true);
    trayMenu->addAction(trayMenuActiver);
    connect(trayMenuActiver, SIGNAL(toggled(bool)), this, SLOT(on_activer(bool)));
    trayMenuActiver->setChecked(Parametres::isActive());

    trayMenu->addSeparator();
    QMenu* menuIntervalle = trayMenu->addMenu("Intervalle");
    QMenu* menuPosition = trayMenu->addMenu("Position");
    QMenu* menuEmplacements = trayMenu->addMenu("Emplacements");
    trayMenu->addSeparator();

    QAction* trayMenuQuitter = new QAction("Quitter", trayMenu);
    trayMenu->addAction(trayMenuQuitter);
    connect(trayMenuQuitter, SIGNAL(triggered()), qApp, SLOT(quit()));

    // Sous-menu intervalle de changement
    int timer = Parametres::getIntervalle();
    bool builtIn = false;
    QActionGroup* groupeIntervalle = new QActionGroup(menuIntervalle);
    QAction* intervalle = new QAction("30 secondes", trayMenu);
    intervalle->setActionGroup(groupeIntervalle);
    intervalle->setCheckable(true);
    intervalle->setChecked(timer == 30 && (builtIn = true));
    intervalle->setProperty("timer", 30);
    intervalle->setProperty("perso", false);
    menuIntervalle->addAction(intervalle);

    intervalle = new QAction("1 minute", trayMenu);
    intervalle->setActionGroup(groupeIntervalle);
    intervalle->setCheckable(true);
    intervalle->setChecked(timer == 60 && (builtIn = true));
    intervalle->setProperty("timer", 60);
    intervalle->setProperty("perso", false);
    menuIntervalle->addAction(intervalle);

    intervalle = new QAction("2 minutes", trayMenu);
    intervalle->setActionGroup(groupeIntervalle);
    intervalle->setCheckable(true);
    intervalle->setChecked(timer == 120 && (builtIn = true));
    intervalle->setProperty("timer", 120);
    intervalle->setProperty("perso", false);
    menuIntervalle->addAction(intervalle);

    intervalle = new QAction("5 minutes", trayMenu);
    intervalle->setActionGroup(groupeIntervalle);
    intervalle->setCheckable(true);
    intervalle->setChecked(timer == 300 && (builtIn = true));
    intervalle->setProperty("timer", 300);
    intervalle->setProperty("perso", false);
    menuIntervalle->addAction(intervalle);

    intervalle = new QAction("10 minutes", trayMenu);
    intervalle->setActionGroup(groupeIntervalle);
    intervalle->setCheckable(true);
    intervalle->setChecked(timer == 600 && (builtIn = true));
    intervalle->setProperty("timer", 600);
    intervalle->setProperty("perso", false);
    menuIntervalle->addAction(intervalle);

    intervalle = new QAction("Personnalis", trayMenu);
    intervalle->setActionGroup(groupeIntervalle);
    intervalle->setCheckable(true);
    intervalle->setChecked(!builtIn);
    intervalle->setProperty("timer", timer);
    intervalle->setProperty("perso", true);
    menuIntervalle->addAction(intervalle);

    connect(groupeIntervalle, SIGNAL(triggered(QAction*)), this, SLOT(on_changerIntervalle(QAction*)));

    // Sous-menu position
    Parametres::STYLE style = Parametres::getStyle();
    QActionGroup* groupePosition = new QActionGroup(menuPosition);
    QAction* position = new QAction("tirer", trayMenu);
    position->setActionGroup(groupePosition);
    position->setCheckable(true);
    position->setChecked(style == Parametres::STRETCHED);
    position->setProperty("position", Parametres::STRETCHED);
    menuPosition->addAction(position);

    position = new QAction("Mosaque", trayMenu);
    position->setActionGroup(groupePosition);
    position->setCheckable(true);
    position->setChecked(style == Parametres::TILED);
    position->setProperty("position", Parametres::TILED);
    menuPosition->addAction(position);

    position = new QAction("Centrer", trayMenu);
    position->setActionGroup(groupePosition);
    position->setCheckable(true);
    position->setChecked(style == Parametres::CENTERED);
    position->setProperty("position", Parametres::CENTERED);
    menuPosition->addAction(position);

    position = new QAction("Ajuster", trayMenu);
    position->setActionGroup(groupePosition);
    position->setCheckable(true);
    position->setChecked(style == Parametres::ADJUST);
    position->setProperty("position", Parametres::ADJUST);
    menuPosition->addAction(position);

    position = new QAction("Remplissage", trayMenu);
    position->setActionGroup(groupePosition);
    position->setCheckable(true);
    position->setChecked(style == Parametres::FILL);
    position->setProperty("position", Parametres::FILL);
    menuPosition->addAction(position);

    connect(groupePosition, SIGNAL(triggered(QAction*)), this, SLOT(on_changerStyle(QAction*)));

    QActionGroup* groupeEmplacements = new QActionGroup(menuEmplacements);
    groupeEmplacements->setExclusive(false);

    for (int i = 0; i < emplacements->rowCount(); i++) {
        QAction* emplacement = new QAction(emplacements->getFiltre(i) + " dans " + emplacements->getPath(i), trayMenu);
        emplacement->setActionGroup(groupeEmplacements);
        emplacement->setCheckable(true);
        emplacement->setChecked(emplacements->getSelection(i));
        emplacement->setProperty("index", i);
        menuEmplacements->addAction(emplacement);
    }
    connect(groupeEmplacements, SIGNAL(triggered(QAction*)), this, SLOT(on_changerEmplacement(QAction*)));
    menuEmplacements->addSeparator();

    QAction* modifierEmplacements = new QAction("Modifier la liste", menuEmplacements);
    menuEmplacements->addAction(modifierEmplacements);
    connect(modifierEmplacements, SIGNAL(triggered()), this, SLOT(show()));

    trayIcon->setContextMenu(trayMenu);
}

void WallpaperRotator2::on_trayIconAction(QSystemTrayIcon::ActivationReason reason) {
    switch(reason) {
        case QSystemTrayIcon::Trigger:
            rotator->changerWallpaper();
            break;
        default:
            break;
    }
}

void WallpaperRotator2::on_changerIntervalle(QAction* action) {
    int timer = 0;
    if (action->property("perso").toBool()) {
        IntervallePerso dialogue;
        dialogue.setIntervalle(action->property("timer").toInt());
        dialogue.exec();
        timer = dialogue.getIntervalle();
        action->setProperty("timer", timer);
    }
    else {
        timer = action->property("timer").toInt();
    }
    rotator->setTimer(timer);
}

void WallpaperRotator2::on_changerStyle(QAction* action) {
    Parametres::STYLE position = (Parametres::STYLE) action->property("position").toInt();
    Parametres::setStyle(position);
}

void WallpaperRotator2::on_ajouterEmplacement() {
    QFileDialog dialogueRepertoire(this, "Slectionnez un rpertoire", QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation));
    dialogueRepertoire.setFileMode(QFileDialog::Directory);
    dialogueRepertoire.setOption(QFileDialog::ShowDirsOnly);
    if (dialogueRepertoire.exec() == QDialog::Accepted) {
        QString path = dialogueRepertoire.directory().path();
        Filtre dialogueFiltre;
        dialogueFiltre.exec();
        emplacements->ajouterEmplacement(path, dialogueFiltre.getFiltre());
        Parametres::setEmplacement(emplacements->rowCount() - 1, path, dialogueFiltre.getFiltre(), false);
        if (emplacements->rowCount() > 0) {
            ui.boutonSupprimer->setEnabled(true);
        }
        creerTrayMenu();
    }
}

void WallpaperRotator2::on_supprimerEmplacement() {
    QModelIndex index = ui.tableView->selectionModel()->currentIndex();
    emplacements->supprimerEmplacement(index.row());
    Parametres::deleteEmplacement(emplacements->rowCount());
    for (int i = index.row(); i < emplacements->rowCount(); i++) {
        Parametres::setEmplacement(i, emplacements->getPath(i), emplacements->getFiltre(i), emplacements->getSelection(i));
    }
    if (emplacements->rowCount() == 0) {
        ui.boutonSupprimer->setEnabled(false);
    }
    creerTrayMenu();
}

void WallpaperRotator2::on_activer(bool activer) {
    Parametres::setActive(activer);
    if (activer) {
        rotator->demarrer();
    }
    else {
        rotator->arreter();
    }
}

void WallpaperRotator2::on_changerEmplacement(QAction* action) {
    int i = action->property("index").toInt();
    emplacements->setSelection(i, action->isChecked());
    Parametres::setEmplacement(i, emplacements->getPath(i), emplacements->getFiltre(i), emplacements->getSelection(i));
    rotator->listerFichiers(emplacements);
}
